/*-
 *  Copyright 2020 Beuth Hochschule für Technik Berlin, Hochschule für Technik Stuttgart
 * 
 *  This file is part of CityDoctor2.
 *
 *  CityDoctor2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CityDoctor2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with CityDoctor2.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.hft.stuttgart.citydoctor2.utils;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

public class Copy {

	private static final Logger logger = LogManager.getLogger(Copy.class);

	private Copy() {
	}

	@SuppressWarnings("unchecked")
	public static <T extends Copyable> T copy(T original) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			ObjectOutputStream oos = new ObjectOutputStream(out);
			oos.writeObject(original);
			oos.flush();
			oos.close();
			byte[] byteArray = out.toByteArray();
			ByteArrayInputStream in = new ByteArrayInputStream(byteArray);
			ObjectInputStream ois = new ObjectInputStream(in);
			return (T) ois.readObject();
		} catch (IOException | ClassNotFoundException e) {
			logger.log(Level.ERROR, e.getMessage(), e);
		}
		throw new IllegalStateException();

	}

}
