package de.hft.stuttgart.citydoctor2.zip;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class CityGmlZipInputStream implements AutoCloseable {

    private ZipFile zip;
    private ZipEntry zipEntry;
    private boolean closed = false;
    private static final long MB = 1024 * 1024L;

    public CityGmlZipInputStream(CityGmlZipEntry entry) throws IOException {
        CityGmlZipArchive archive = entry.getArchive();
        zip = new ZipFile(archive.getArchivePath().toFile());
        zipEntry = zip.getEntry(entry.getFileName());
    }

    public InputStream getInputStream() throws IOException {
        if (closed){
            throw new IOException("Stream closed");
        }
        return zip.getInputStream(zipEntry);
    }

    public long getFileSize(long memoryLimit) throws IOException {
        if (closed){
            throw new IOException("Stream closed");
        }
        if (zipEntry.getSize() != -1){
            return (long) Math.ceil((double) zipEntry.getSize() / MB);
        }
        InputStream is = this.getInputStream();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        for(int i = is.read(); i != -1; i=is.read()) {
            baos.write(i);
            if ((baos.size() / MB) + 1 > memoryLimit) {
                //Entry exceeds memory limit
                return -1L;
            }
        }
        // end of stream reached without exceeding memory limit
        return (long) Math.ceil((double) baos.size() / MB);
    }

    @Override
    public void close() throws IOException {
        zip.close();
        closed = true;
    }
}
